
#include <iostream>
using std::cout;
using std::cin;
using std::cerr;
using std::endl;

#include <string>
using std::string;

#include "RefCount.h"

#include "ResolveNameToObject.h"

int main (int argc, char **argv) 
{
   try 
   {
	
      CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "omniORB3");

      CORBA::Object_var obj = ResolveNameToObject(orb, "JetByte Test", "NamedCounterFactory");

      NamedCounterFactory_var counterFactory = NamedCounterFactory::_narrow(obj);

      if (!CORBA::is_nil(counterFactory))
      {
         bool ok = false;

         if (argc > 1)
         {
            if (argc == 2 && 0 == stricmp("list", argv[1]))
            {
               NamedCounterFactory::counterSeq_var seq = counterFactory->GetAll();

               cout << "List of named counters:" << endl;

               for (size_t i = 0; i < seq->length(); ++i)
               {
                  cout << seq[i]->name() << " - " << seq[i]->value() << endl;

                  seq[i]->Release();
               }

               cout << "Done." << endl;

               ok = true;
            }
            else if (argc == 3)
            {
               if (0 == stricmp("add", argv[1]))
               {
                  counterFactory->AddNamedCounter(argv[2], 0);

                  cout << "Counter " << argv[2] << " added with a value of 0" << endl;
                  
                  ok = true;
               }
               else if (0 == stricmp("del", argv[1]))
               {
                  counterFactory->RemoveNamedCounter(argv[2]);

                  cout << "Counter " << argv[2] << " deleted" << endl;
   
                  ok = true;
               }
            }
            else if (argc == 4)
            {
               if (0 == stricmp("change", argv[1]))
               {
                  NamedCounter_var counter = counterFactory->GetNamedCounter(argv[2]);
               
                  const long initialVal = counter->value();
                  const long newVal = initialVal + atoi(argv[3]);

                  counter->value(newVal);

                  counter->Release();

                  cout << "Counter " << argv[2] << " initial value = " << initialVal << " new value = " << newVal << endl;

                  ok = true;
               }
               else if (0 == stricmp("hold", argv[1]))
               {
                  NamedCounter_var counter = counterFactory->GetNamedCounter(argv[2]);
  
                  cout << "Counter " << argv[2] << " held" << endl;
   
                  Sleep(1000*atoi(argv[3]));

                  long value = counter->value();

                  counter->Release();

                  cout << "Counter " << argv[2] << " value = " << value << endl;

                  ok = true;
               }
            }
         }

         if (!ok)
         {
            cout << "usage: client list          - list all counters and values" << endl;
            cout << "       client add <counter> - add a new counter" << endl;
            cout << "       client del <counter> - remove a counter" << endl;
            cout << "       client change <counter> <value> - change the counter by value" << endl;
            cout << "       client hold <counter> <secs> - obtain the counter and hold onto it for secs seconds" << endl;
         }
      }
      else
      {
         cout << "Failed to narrow counter factory" << endl;
      }

      orb->destroy();
   }
   catch (NamedCounterFactory::CounterExists &e)
   {
      cerr << "Counter \"" << (char*)e.name << "\" already exists!" << endl;
   }
   catch (NamedCounterFactory::NoCounterExists &e)
   {
      cerr << "Counter \"" << (char*)e.name << "\" doesn't exist!" << endl;
   }
   catch(CORBA::COMM_FAILURE& ) 
   {
      cerr << "Caught system exception COMM_FAILURE -- unable to contact the object." << endl;
   }
   catch(CORBA::SystemException&) 
   {
      cerr << "Caught CORBA::SystemException." << endl;
   }
   catch(CORBA::Exception&) 
   {
      cerr << "Caught CORBA::Exception." << endl;
   }
   catch(omniORB::fatalException& fe) 
   {
      cerr << "Caught omniORB::fatalException:" << endl;
      cerr << "  file: " << fe.file() << endl;
      cerr << "  line: " << fe.line() << endl;
      cerr << "  mesg: " << fe.errmsg() << endl;
   }
   catch(...) 
   {
      cerr << "Caught unknown exception." << endl;
   }

   return 0;
}


